@@ -4,41 +4,24 @@ use strict;
 
 use vars qw($VERSION);
 
-$VERSION = '0.33_01';
-$VERSION = eval $VERSION;
+$VERSION = '0.1';
 
 use base qw(Class::Container);
 
-use Apache::Session 1.81;
+use Apache::Session 1.6;
 
 use Exception::Class ( 'Apache::Session::Wrapper::Exception::NonExistentSessionID' =>
 		       { description => 'A non-existent session id was used',
 			 fields => [ 'session_id' ] },
                        'Apache::Session::Wrapper::Exception::Params' =>
-		       { description => 'An invalid parameter or set of parameters was given',
+		       { description => 'A non-existent session id was used',
                          alias => 'param_error' },
 		     );
 
 use Params::Validate 0.70;
-use Params::Validate qw( validate SCALAR UNDEF BOOLEAN ARRAYREF OBJECT );
+use Params::Validate qw( validate SCALAR UNDEF BOOLEAN OBJECT );
 Params::Validate::validation_options( on_fail => sub { param_error( join '', @_ ) } );
 
-use Scalar::Util ();
-
-
-my $MOD_PERL = _find_mp_version();
-sub _find_mp_version
-{
-    return 0 unless $ENV{MOD_PERL};
-
-    return
-        ( $ENV{MOD_PERL} =~ /(?:1\.9|2\.\d)/
-          ? 2
-          : 1
-        );
-}
-
-my @HeaderMethods = qw( err_headers_out headers_out );
 
 my %params =
     ( always_write =>
@@ -53,13 +36,13 @@ my %params =
 
       param_name =>
       { type => SCALAR,
-        optional => 1,
+	default => undef,
         depends => 'param_object',
 	descr => 'Name of the parameter to use for session tracking' },
 
       param_object =>
       { type => OBJECT,
-        optional => 1,
+	default => undef,
         can  => 'param',
 	descr => 'Object which has a "param" method, to be used for getting the session id from a query string or POST argument' },
 
@@ -70,7 +53,7 @@ my %params =
 
       cookie_name =>
       { type => SCALAR,
-	default => 'Apache-Session-Wrapper-cookie',
+	default => 'MasonX-Request-WithApacheSession-cookie',
 	descr => 'Name of cookie used by this module' },
 
       cookie_expires =>
@@ -80,7 +63,7 @@ my %params =
 
       cookie_domain =>
       { type => UNDEF | SCALAR,
-        optional => 1,
+	default => undef,
 	descr => 'Domain parameter for cookies' },
 
       cookie_path =>
@@ -101,9 +84,9 @@ my %params =
       header_object =>
       { type => OBJECT,
         callbacks =>
-        { 'has a method to set headers' =>
-          sub { grep { $_[0]->can($_) } @HeaderMethods } },
-        optional => 1,
+        { 'header method' =>
+          sub { $_[0]->can('err_header_out') || $_[0]->can('header_out' ) } },
+        default => undef,
         descr => 'An object that can be used to send cookies with' },
 
       class =>
@@ -117,7 +100,7 @@ my %params =
 
       user_name =>
       { type => UNDEF | SCALAR,
-        optional => 1,
+	default => undef,
 	descr => 'The user name to be used when connecting to a database' },
 
       password =>
@@ -137,7 +120,7 @@ my %params =
 
       lock_user_name =>
       { type => UNDEF | SCALAR,
-        optional => 1,
+        default => undef,
 	descr => 'The user name to be used when connecting to a database' },
 
       lock_password =>
@@ -167,12 +150,12 @@ my %params =
 
       directory =>
       { type => SCALAR,
-	optional => 1,
+	default => undef,
 	descr => 'A directory to use when storing sessions' },
 
       lock_directory =>
       { type => SCALAR,
-	optional => 1,
+	default => undef,
 	descr => 'A directory to use for locking when storing sessions' },
 
       file_name =>
@@ -230,20 +213,16 @@ my %params =
 	optional => 1,
 	descr => 'Path used by Apache::Session::PHP' },
 
-      session_id =>
-      { type => SCALAR,
-	optional => 1,
-	descr => 'Try this session id first when making a session' },
     );
 
 # What set of parameters are required for each session class.
 # Multiple array refs represent multiple possible sets of parameters
 my %ApacheSessionParams =
     ( Flex     => [ [ qw( store lock generate serialize ) ] ],
-      MySQL    => [ [ qw( data_source user_name
-                          lock_data_source lock_user_name ) ],
+      MySQL    => [ [ qw( data_source user_name password
+                          lock_data_source lock_user_name lock_password ) ],
 		    [ qw( handle lock_handle ) ] ],
-      Postgres => [ [ qw( data_source user_name commit ) ],
+      Postgres => [ [ qw( data_source user_name password commit ) ],
 		    [ qw( handle commit ) ] ],
       File     => [ [ qw( directory lock_directory ) ] ],
       DB_File  => [ [ qw( file_name lock_directory ) ] ],
@@ -255,24 +234,23 @@ my %ApacheSessionParams =
     ( $ApacheSessionParams{Postgres} ) x 3;
 
 my %OptionalApacheSessionParams =
-    ( MySQL    => [ [ qw( table_name password lock_password ) ] ],
-      Postgres => [ [ qw( table_name password ) ] ],
-      Informix => [ [ qw( table_name password ) ] ],
-      Oracle   => [ [ qw( long_read_len table_name password ) ] ],
-      Sybase   => [ [ qw( textsize table_name password ) ] ],
+    ( MySQL    => [ [ qw( table_name ) ] ],
+      Postgres => [ [ qw( table_name ) ] ],
+      Informix => [ [ qw( long_read_len table_name ) ] ],
+      Oracle   => [ [ qw( long_read_len table_name ) ] ],
+      Sybase   => [ [ qw( textsize table_name ) ] ],
     );
 
 my %ApacheSessionFlexParams =
     ( store =>
-      { MySQL    => [ [ qw( data_source user_name ) ],
+      { MySQL    => [ [ qw( data_source user_name password ) ],
 		      [ qw( handle ) ] ],
 	Postgres => $ApacheSessionParams{Postgres},
 	File     => [ [ qw( directory ) ] ],
 	DB_File  => [ [ qw( file_name ) ] ],
-	PHP      => $ApacheSessionParams{PHP},
       },
       lock =>
-      { MySQL     => [ [ qw( lock_data_source lock_user_name ) ],
+      { MySQL     => [ [ qw( lock_data_source lock_user_name lock_password ) ],
 		       [ qw( lock_handle ) ] ],
 	File      => [ [ ] ],
 	Null      => [ [ ] ],
@@ -288,73 +266,19 @@ my %ApacheSessionFlexParams =
 	Base64   => [ [ ] ],
 	Sybase   => [ [ ] ],
 	UUEncode => [ [ ] ],
-	PHP      => [ [ ] ],
       },
     );
 
+__PACKAGE__->valid_params(%params);
+
 @{ $ApacheSessionFlexParams{store} }{ qw( Informix Oracle Sybase ) } =
     ( $ApacheSessionFlexParams{store}{Postgres} ) x 3;
 
 my %OptionalApacheSessionFlexParams =
-    ( store => { map { $_ => $OptionalApacheSessionParams{$_} }
-                 qw( MySQL Postgres Informix Oracle Sybase ) },
+    ( Sybase => { store => [ qw( textsize ) ] },
+      Oracle => { store => [ qw( long_read_len ) ] },
     );
 
-sub _SetValidParams {
-    my $class = shift;
-
-    my %extra;
-    for my $hash ( \%ApacheSessionParams,
-                   \%OptionalApacheSessionParams,
-                   @ApacheSessionFlexParams{ qw( store lock generate serialize ) },
-                   @OptionalApacheSessionFlexParams{ qw( store lock generate serialize ) },
-                 )
-    {
-        for my $p ( map { @$_ } map { @$_ } values %$hash )
-        {
-            my $h;
-            if ( ref $p ) {
-                # we assume its a hash of names/parameter specifications
-                $h = $p;
-            } elsif (!$params{$p}) {
-                # its a new parameter defined by a scalar, default to SCALAR value
-                $h = { $p => { optional => 1, type => SCALAR } };
-            } else {
-                # its a scalar option we already know.
-                next;
-            }
-            # now expand the options
-            foreach my $name (keys %$h) {
-                next if $params{$name};
-                $extra{$p} = $h->{$name};
-            }
-        }
-    }
-
-    $class->valid_params( %extra, %params );
-    $class->SetStudlyForms();
-}
-__PACKAGE__->_SetValidParams();
-
-my %StudlyForm;
-sub SetStudlyForms
-{
-    %StudlyForm =
-        ( map { $_ => _studly_form($_) }
-          map { ref $_ ? @$_ :$_ }
-          map { @$_ }
-          ( values %ApacheSessionParams ),
-          ( values %OptionalApacheSessionParams ),
-          ( map { values %{ $ApacheSessionFlexParams{$_} } }
-            keys %ApacheSessionFlexParams ),
-          ( map { values %{ $OptionalApacheSessionFlexParams{$_} } }
-            keys %OptionalApacheSessionFlexParams ),
-        );
-
-    # why Apache::Session does this I do not know
-    $StudlyForm{textsize} = 'textsize';
-}
-
 sub _studly_form
 {
     my $string = shift;
@@ -362,66 +286,27 @@ sub _studly_form
     return $string;
 }
 
-sub RegisterClass {
-    my $class = shift;
-    my %p = validate( @_, { name => { type => SCALAR },
-                            required => { type => SCALAR | ARRAYREF, default => [ [ ] ] },
-                            optional => { type => SCALAR | ARRAYREF, default => [ ] },
-                          },
-                    );
+my %StudlyForm =
+    ( map { $_ => _studly_form($_) }
+      map { ref $_ ? @$_ :$_ }
+      map { @$_ }
+      ( values %ApacheSessionParams ),
+      ( values %OptionalApacheSessionParams ),
+      ( map { values %{ $ApacheSessionFlexParams{$_} } }
+	keys %ApacheSessionFlexParams ),
+      ( map { values %{ $OptionalApacheSessionFlexParams{$_} } }
+	keys %OptionalApacheSessionFlexParams ),
+    );
 
-    $p{name} =~ s/^Apache::Session:://;
-
-    $ApacheSessionParams{ $p{name} } =
-        ( ref $p{required}
-          ? $p{required}
-          : $ApacheSessionParams{ $p{required} }
-        );
-
-    $OptionalApacheSessionParams{ $p{name} } =
-        ( ref $p{optional}
-          ? [ $p{optional} ]
-          : $OptionalApacheSessionParams{ $p{optional} }
-        );
-
-    $class->_SetValidParams();
-}
-
-sub RegisterFlexClass {
-    my $class = shift;
-    my %p = validate( @_, { type => { type => SCALAR,
-                                      regex => qr/^(?:store|lock|generate|serialize)/,
-                                    },
-                            name => { type => SCALAR },
-                            required => { type => SCALAR | ARRAYREF, default => [ [ ] ] },
-                            optional => { type => SCALAR | ARRAYREF, default => [ ]  },
-                          },
-                    );
-
-    $p{name} =~ s/^Apache::Session:://;
-    $p{name} =~ s/^\Q$p{type}\E:://i;
-
-    $ApacheSessionFlexParams{ $p{type} }{ $p{name} } =
-        ( ref $p{required}
-          ? $p{required}
-          : $ApacheSessionFlexParams{ $p{type} }{ $p{required} }
-        );
-
-    $OptionalApacheSessionFlexParams{ $p{type} }{ $p{name} } =
-        ( ref $p{optional}
-          ? [ $p{optional} ]
-          : $OptionalApacheSessionFlexParams{ $p{type} }{ $p{optional} }
-        );
-
-    $class->_SetValidParams();
-}
+# why Apache::Session does this I do not know
+$StudlyForm{textsize} = 'textsize';
 
 sub new
 {
-    my $class = shift;
-    my %p = @_;
 
-    my $self = $class->SUPER::new(%p);
+    my $class = shift;
+
+    my $self = $class->SUPER::new(@_);
 
     $self->_check_session_params;
     $self->_set_session_params;
@@ -429,17 +314,13 @@ sub new
     if ( $self->{use_cookie} && ! ( $ENV{MOD_PERL} || $self->{header_object} ) )
     {
         param_error
-            "The header_object parameter is required in order to use cookies outside of mod_perl";
+            "The header_object parameter is required to use cookies outside of mod_perl";
     }
 
-    my $session_class = "Apache::Session::$self->{session_class_piece}";
-    unless ( $session_class->can('TIEHASH') )
-    {
-        eval "require $session_class";
-        die $@ if $@;
-    }
+    eval "require Apache::Session::$self->{session_class_piece}";
+    die $@ if $@;
 
-    $self->_make_session( $p{session_id} );
+    $self->_make_session;
 
     $self->_bake_cookie
         if $self->{use_cookie} && ! $self->{cookie_is_baked};
@@ -457,19 +338,23 @@ sub _check_session_params
     my $sets = $ApacheSessionParams{ $self->{session_class_piece} }
 	or param_error "Invalid session class: $self->{class}";
 
-    $self->_check_sets( $sets, 'session', $self->{class} )
-        if grep { @$_ } @$sets;
+    my $complete = $self->_check_sets($sets);
+
+    param_error "Not all of the required parameters for your chosen session class ($self->{class}) were provided."
+	unless $complete;
 
     if ( $self->{session_class_piece} eq 'Flex' )
     {
 	foreach my $key ( keys %ApacheSessionFlexParams )
 	{
-	    my $subclass = $self->{$key};
+	    my $subclass = $self->{"$key"};
 	    my $sets = $ApacheSessionFlexParams{$key}{$subclass}
 		or param_error "Invalid class for $key: $self->{$key}";
 
-            $self->_check_sets( $sets, $key, $subclass )
-                if grep { @$_ } @$sets;
+	    my $complete = $self->_check_sets($sets);
+
+	    param_error "Not all of the required parameters for your chosen $key class ($subclass) were provided."
+		unless $complete;
 	}
     }
 }
@@ -478,21 +363,14 @@ sub _check_sets
 {
     my $self = shift;
     my $sets = shift;
-    my $type = shift;
-    my $class = shift;
 
-    my @missing;
     foreach my $set (@$sets)
     {
-        my @matched = grep { exists $self->{$_} } @$set;
-
-        return if @matched == @$set;
-
-        @missing = grep { ! exists $self->{$_} } @$set;
+	return 1
+	    if ( grep { exists $self->{"$_"} } @$set ) == @$set;
     }
 
-    param_error "Some or all of the required parameters for your chosen $type class ($class) were provided."
-                . "  The following parameters were missing: @missing\n";
+    return 0;
 }
 
 sub _set_session_params
@@ -514,7 +392,7 @@ sub _set_session_params
     {
 	foreach my $key ( keys %ApacheSessionFlexParams )
 	{
-	    my $subclass = $self->{$key};
+	    my $subclass = $self->{"$key"};
 	    $params{ $StudlyForm{$key} } = $subclass;
 
 	    $self->_sets_to_params
@@ -529,55 +407,24 @@ sub _set_session_params
 
     $self->{params} = \%params;
 
-    $self->_set_cookie_fields
-        if $self->{use_cookie};
-}
-
-sub _set_cookie_fields
-{
-    my $self = shift;
-
-    my $cookie_class;
-    if ($MOD_PERL)
+    if ( $self->{use_cookie} )
     {
-        $cookie_class =
-            $MOD_PERL == 2 ? 'Apache2::Cookie' : 'Apache::Cookie';
+        if ( $ENV{MOD_PERL} )
+        {
+            eval { require Apache::Cookie; Apache::Cookie->can('bake'); };
+            unless ($@)
+            {
+                $self->{cookie_class} = 'Apache::Cookie';
+                $self->{new_cookie_args} = [ Apache->request ];
+            }
+        }
 
-        eval "require $cookie_class"
-            unless $cookie_class->can('new');
-    }
-
-    unless ( $cookie_class && $cookie_class->can('new' ) )
-    {
-        require CGI::Cookie;
-        $cookie_class = 'CGI::Cookie';
-    }
-
-    $self->{cookie_class} = $cookie_class;
-
-    if ( $self->{cookie_class} eq 'CGI::Cookie' )
-    {
-        $self->{new_cookie_args} = [];
-        $self->{fetch_cookie_args} = [];
-    }
-    else
-    {
-        $self->{new_cookie_args} =
-            [ $MOD_PERL == 2
-              ? Apache2::RequestUtil->request
-              : Apache->request
-            ];
-
-        $self->{fetch_cookie_args} =
-            ( $MOD_PERL == 2
-              ? $self->{new_cookie_args}
-              : []
-            );
-        $self->{bake_cookie_args} =
-            ( $MOD_PERL == 2
-              ? $self->{new_cookie_args}
-              : []
-            );
+        unless ( $self->{cookie_class} )
+        {
+            require CGI::Cookie;
+            $self->{cookie_class} = 'CGI::Cookie';
+            $self->{new_cookie_args} = [];
+        }
     }
 }
 
@@ -591,10 +438,10 @@ sub _sets_to_params
     {
 	foreach my $key (@$set)
 	{
-	    if ( exists $self->{$key} )
+	    if ( exists $self->{"$key"} )
 	    {
 		$params->{ $StudlyForm{$key} } =
-		    $self->{$key};
+		    $self->{"$key"};
 	    }
 	}
     }
@@ -603,10 +450,15 @@ sub _sets_to_params
 sub _make_session
 {
     my $self = shift;
-    my $session_id = shift;
+    my %p = validate( @_,
+		      { session_id =>
+			{ type => SCALAR,
+                          optional => 1,
+			},
+		      } );
 
     return if
-        defined $session_id && $self->_try_session_id( $session_id );
+        defined $p{session_id} && $self->_try_session_id( $p{session_id} );
 
     my $id = $self->_get_session_id;
     return if defined $id && $self->_try_session_id($id);
@@ -645,26 +497,6 @@ sub _get_session_id_from_args
     return $self->{param_object}->param( $self->{param_name} );
 }
 
-sub _get_session_id_from_cookie
-{
-    my $self = shift;
-
-    if ( $MOD_PERL == 2 )
-    {
-        my $jar = Apache2::Cookie::Jar->new( @{ $self->{fetch_cookie_args} } );
-        my $c   = $jar->cookies( $self->{cookie_name} );
-        return $c->value if $c;
-    }
-    else
-    {
-        my %c = $self->{cookie_class}->fetch( @{ $self->{fetch_cookie_args} } );
-
-        return $c{ $self->{cookie_name} }->value
-            if exists $c{ $self->{cookie_name} };
-    }
-    return undef;
-}
-
 sub _try_session_id
 {
     my $self = shift;
@@ -683,7 +515,7 @@ sub _try_session_id
                 $session_id, $self->{params};
 	};
 
-        if ( $@ || ! tied %s || ! $s{_session_id} )
+        if ($@)
         {
             $self->_handle_tie_error( $@, $session_id );
             return;
@@ -700,13 +532,25 @@ sub _try_session_id
     return 1;
 }
 
+sub _get_session_id_from_cookie
+{
+    my $self = shift;
+
+    my %c = $self->{cookie_class}->fetch;
+
+    return $c{ $self->{cookie_name} }->value
+        if exists $c{ $self->{cookie_name} };
+
+    return undef;
+}
+
 sub _handle_tie_error
 {
     my $self = shift;
     my $err = shift;
     my $session_id = shift;
 
-    if ( $err =~ /Object does not exist/ && defined $session_id )
+    if ( $err =~ /Object does not exist/ )
     {
         return if $self->{allow_invalid_id};
 
@@ -716,9 +560,7 @@ sub _handle_tie_error
     }
     else
     {
-        my $error =
-            $err ? $err : "Tying to Apache::Session::$self->{session_class_piece} failed but did not throw an exception";
-        die $error;
+        die $@;
     }
 }
 
@@ -727,7 +569,6 @@ sub _bake_cookie
     my $self = shift;
 
     my $expires = shift || $self->{cookie_expires};
-    $expires = undef if defined $expires && $expires =~ /^session$/i;
 
     my $domain = $self->{cookie_domain};
 
@@ -735,39 +576,27 @@ sub _bake_cookie
         $self->{cookie_class}->new
             ( @{ $self->{new_cookie_args} },
               -name    => $self->{cookie_name},
-              # Apache2::Cookie will return undef if we pass undef for
-              # -value.
-              -value   => ( $self->{session_id} || '' ),
-              ( defined $expires
-                ? ( -expires => $expires )
-                : ()
-              ),
-              ( defined $domain
-                ? ( -domain  => $domain )
-                : ()
+              -value   => $self->{session_id},
+              -expires => $expires,
+              ( defined $domain ?
+                ( -domain  => $domain ) :
+                ()
               ),
               -path    => $self->{cookie_path},
               -secure  => $self->{cookie_secure},
             );
 
-    # If not running under mod_perl, CGI::Cookie->bake() will call
-    # print() to send a cookie header right now, which may not be what
-    # the user wants.
-    if ( $cookie->can('bake') && ! $cookie->isa('CGI::Cookie') )
+    if ( $cookie->can('bake') )
     {
-        $cookie->bake( @{ $self->{bake_cookie_args} } );
+        # Apache::Cookie
+        $cookie->bake;
     }
     else
     {
         my $header_object = $self->{header_object};
-        for my $meth (@HeaderMethods)
-        {
-            if ( $header_object->can($meth) )
-            {
-                $header_object->$meth->add( 'Set-Cookie' => $cookie );
-                last;
-            }
-        }
+        my $meth = $header_object->can('err_header_out') ? 'err_header_out' : 'header_out';
+
+        $header_object->$meth( 'Set-Cookie' => $cookie );
     }
 
     # always set this even if we skipped actually setting the cookie
@@ -780,16 +609,10 @@ sub _bake_cookie
 sub session
 {
     my $self = shift;
-    my %p = validate( @_,
-		      { session_id =>
-			{ type => SCALAR,
-                          optional => 1,
-			},
-		      } );
 
-    if ( ! $self->{session} || %p )
+    if ( ! $self->{session} || @_ )
     {
-        $self->_make_session( $p{session_id} );
+        $self->_make_session(@_);
 
         $self->_bake_cookie
             if $self->{use_cookie} && ! $self->{cookie_is_baked};
@@ -879,17 +702,10 @@ This class provides the following public methods:
 
 This method creates a new C<Apache::Session::Wrapper> object.
 
-If the parameters you provide are not correct (wrong type, missing
-parameters, etc.), this method throws an
-C<Apache::Session::Wrapper::Exception::Params> exception.  You can
-treat this exception as a string if you want.
-
 =item * session
 
 This method returns a hash tied to the C<Apache::Session> class.
 
-This method accepts an optional "session_id" parameter.
-
 =item * delete_session
 
 This method deletes the existing session from persistent storage.  If
@@ -905,81 +721,6 @@ simply passed through to C<Apache::Session>.  For this reason, you are
 advised to familiarize yourself with the C<Apache::Session>
 documentation before attempting to configure this module.
 
-You can also register C<Apache::Session> classes, or the classes used
-for doing the work in C<Apache::Session::Flex>. See L<REGISTERING
-CLASSES> for details.
-
-=head2 Supported Classes
-
-The following classes are already supported and do not require
-registration:
-
-=over 4
-
-=item * Apache::Session::MySQL
-
-=item * Apache::Session::Postgres
-
-=item * Apache::Session::Oracle
-
-=item * Apache::Session::Informix
-
-=item * Apache::Session::Sybase
-
-=item * Apache::Session::File
-
-=item * Apache::Session::DB_File
-
-=item * Apache::Session::PHP
-
-=item * Apache::Session::Flex
-
-=back
-
-The following classes can be used with C<Apache::Session::Flex>:
-
-=over 4
-
-=item * Apache::Session::Store::MySQL
-
-=item * Apache::Session::Store::Postgres
-
-=item * Apache::Session::Store::Informix
-
-=item * Apache::Session::Store::Oracle
-
-=item * Apache::Session::Store::Sybase
-
-=item * Apache::Session::Store::File
-
-=item * Apache::Session::Store::DB_File
-
-=item * Apache::Session::Store::PHP
-
-=item * Apache::Session::Lock::MySQL
-
-=item * Apache::Session::Lock::File
-
-=item * Apache::Session::Lock::Null
-
-=item * Apache::Session::Lock::Semaphore
-
-=item * Apache::Session::Generate::MD5
-
-=item * Apache::Session::Generate::ModUsertrack
-
-=item * Apache::Session::Serialize::Storable
-
-=item * Apache::Session::Serialize::Base64
-
-=item * Apache::Session::Serialize::Sybase
-
-=item * Apache::Session::Serialize::UUEncode
-
-=item * Apache::Session::Serialize::PHP
-
-=back
-
 =head2 Generic Parameters
 
 =over 4
@@ -1005,16 +746,11 @@ This defaults to true.
 If this is true, an attempt to create a session with a session id that
 does not exist in the session storage will be ignored, and a new
 session will be created instead.  If it is false, a
-C<Apache::Session::Wrapper::Exception::NonExistentSessionID> exception
-will be thrown instead.
+C<HTML::Mason::Exception::NonExistentSessionID> exception will be
+thrown instead.
 
 This defaults to true.
 
-=item * session_id  =>  string
-
-Try this session id first and use it if it exist. If the session does
-not exist, it will ignore this parameter and make a new session.
-
 =back
 
 =head2 Cookie-Related Parameters
@@ -1023,14 +759,16 @@ not exist, it will ignore this parameter and make a new session.
 
 =item * use_cookie  =>  boolean
 
-If true, then this module will use one of C<Apache::Cookie>,
-C<Apache2::Cookie> or C<CGI::Cookie> (as appropriate) to set and read
+If true, then this module will use C<Apache::Cookie> to set and read
 cookies that contain the session id.
 
+The cookie will be set again every time the client accesses a Mason
+component unless the C<cookie_resend> parameter is false.
+
 =item * cookie_name  =>  name
 
 This is the name of the cookie that this module will set.  This
-defaults to "Apache-Session-Wrapper-cookie".
+defaults to "MasonX-Request-WithApacheSession-cookie".
 Corresponds to the C<Apache::Cookie> "-name" constructor parameter.
 
 =item * cookie_expires  =>  expiration
@@ -1038,10 +776,6 @@ Corresponds to the C<Apache::Cookie> "-name" constructor parameter.
 How long before the cookie expires.  This defaults to 1 day, "+1d".
 Corresponds to the "-expires" parameter.
 
-As a special case, you can set this value to "session" to have the
-"-expires" parameter set to undef, which gives you a cookie that
-expires at the end of the session.
-
 =item * cookie_domain  =>  domain
 
 This corresponds to the "-domain" parameter.  If not given this will
@@ -1067,12 +801,11 @@ cookie has the effect of updating the expiration time.
 =item * header_object => object
 
 When running outside of mod_perl, you must provide an object to which
-the cookie header can be added.  This object must provide an
-C<err_headers_out()> or C<headers_out()> method.
+the cookie header can be added.  This object must provide either an
+C<err_header_out()> or C<header_out()> method.
 
-Under mod_perl 1, this will default to the object returned by C<<
-Apache->request() >>. Under mod_perl 2 we call C<<
-Apache2::RequestUtil->request() >>
+Under mod_perl, this will default to the object returned by C<<
+Apache->request >>.
 
 =back
 
@@ -1089,6 +822,8 @@ name of the parameter that is checked.
 If you are also using cookies, then the module checks the param object
 I<first>, and then it checks for a cookie.
 
+The session id is available from C<< $m->session->{_session_id} >>.
+
 =item * param_object  =>  object
 
 This should be an object that provides a C<param()> method.  This
@@ -1117,7 +852,7 @@ session modules.
 =item * password  =>  password
 
 Corresponds to the C<Password> parameter passed to the DBI-related
-session modules.  Defaults to undef.
+session modules.
 
 =item * handle =>  DBI handle
 
@@ -1144,7 +879,7 @@ C<Apache::Session::MySQL>.
 =item * lock_password  =>  password
 
 Corresponds to the C<LockPassword> parameter passed to
-C<Apache::Session::MySQL>.  Defaults to undef.
+C<Apache::Session::MySQL>.
 
 =item * lock_handle  =>  DBI handle
 
@@ -1234,116 +969,9 @@ When run under mod_perl, this module attempts to first use
 C<Apache::Cookie> for cookie-handling.  Otherwise it uses
 C<CGI::Cookie> as a fallback.
 
-If it ends up using C<CGI::Cookie> then you must provide a
-"header_object" parameter. This object must have an
-C<err_headers_out()> or C<headers_out()> method. It looks for these
-methods in that order. The method is expected to return an object with
-an API like C<Apache::Table>. It calls C<add()> on the returned method
-to add a "Set-Cookie" header.
-
-=head1 REGISTERING CLASSES
-
-In order to support any C<Apache::Session> subclasses, this module
-provides a simple registration mechanism.
-
-You can register an C<Apache::Session> subclass, or a class intended
-to provide a class that implements something required by
-C<Apache::Session::Flex>.
-
-=head2 Registering a Complete Subclass
-
-This is done by calling C<< Apache::Session::Wrapper->RegisterClass() >>:
-
-  Apache::Session::Wrapper->RegisterClass
-      ( name     => 'MyClass',
-        required => [ [ qw( param1 param2 ) ],
-                      [ qw( param3 param4 ) ] ],
-        optional => [ 'optional_p' ],
-      );
-
-  Apache::Session::Wrapper->RegisterClass
-      ( name     => 'Apache::Session::MyFile',
-        required => 'File',
-        optional => 'File',
-      );
-
-The C<RegisterClass()> method takes the following options:
-
-=over 4
-
-=item * name
-
-This should be the name of the class you are registering. The actual
-class must start with "Apache::Session::", but this part does not need
-to be included when registering the class (it's optional).
-
-=item * required
-
-These are the required parameters for this class.
-
-The value of this parameter can either be a string or a reference to
-an array of array references.
-
-If it is a string, then it identifies an existing C<Apache::Session>
-subclass which is already registered or built-in, like "File" or
-"Postgres".
-
-If it an array reference, then I<that reference> should in turn
-contain one or more array references. Each of those contained
-references represents one set of required parameters. When an
-C<Apache::Session::Wrapper> object is constructed, only one of these
-sets must be passed in. For example:
-
-  required => [ [ qw( p1 p2 ) ],
-                [ qw( p2 p3 p4 ) ] ]
-
-This says that either "p1" and "p2" must be provided, I<or> "p2",
-"p3", and "p4".
-
-If there are no required parameters for this class, then the
-"required" parameter can be omitted.
-
-=item * optional
-
-This specifies optional parameters, and should just be a simple array
-reference.
-
-=back
-
-=head2 Registering a Subclass for Flex
-
-Registering a subclass that can be used with C<Apache::Session::Flex>
-is very similar to registering a complete class:
-
-  Apache::Session::Wrapper->RegisterFlexClass
-      ( name     => 'MyClass',
-        type     => 'Store',
-        required => [ [ qw( param1 param2 ) ],
-                      [ qw( param3 param4 ) ] ],
-        optional => [ 'optional_p' ],
-      );
-
-  Apache::Session::Wrapper->RegisterFlexClass
-      ( name     => 'Apache::Session::Store::MyFile',
-        type     => 'store',
-        required => 'File',
-        optional => 'File',
-      );
-
-The C<RegisterFlexClass()> method has the same parameters as
-C<RegisterClass()>, but it also requires a "type" parameter. This must
-be one of "store", "lock", "generate", or "serialize".
-
-=head1 SUBCLASSING
-
-This class provides a simple hook for subclasses.  Before trying to
-get a session id from the URL or cookie, it calls a method named
-C<_get_session_id()>.  In this class, that method is a no-op, but you
-can override this in a subclass.
-
-This class is a C<Class::Container> subclass, so if you accept
-additional constructor parameters, you should declare them via the
-C<valid_params()> method.
+If it ends up using C<CGI::Cookie> then must provide a "header_object"
+parameter.  The module calls C<err_header_out()> or C<header_out()> on
+the provided object, using the former if it's available.
 
 =head1 SUPPORT
 
@@ -1355,19 +983,10 @@ Please submit bugs to the CPAN RT system at
 http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Apache%3A%3ASession%3A%3AWrapper
 or via email at bug-apache-session-wrapper@rt.cpan.org.
 
-Support questions can be sent to me at my email address, shown below.
+Support questions can be sent to me at my email address, listed below.
 
 =head1 AUTHOR
 
 Dave Rolsky, <autarch@urth.org>
 
-=head1 COPYRIGHT
-
-Copyright (c) 2003-2006 David Rolsky.  All rights reserved.  This
-program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-The full text of the license can be found in the LICENSE file included
-with this module.
-
 =cut
